@@ -1,37 +1,44 @@
-from __future__ import division
-
 from copy import deepcopy
 from difflib import SequenceMatcher
 from lxml import etree
+from math import sqrt
 from xmldiff import utils, actions
 
 
-class Differ(object):
-
-    def __init__(self, F=None, uniqueattrs=None, ratio_mode='fast',
-                 fast_match=False):
+class Differ:
+    def __init__(
+        self,
+        F=None,
+        uniqueattrs=None,
+        ratio_mode="fast",
+        fast_match=False,
+        ignored_attrs=[],
+    ):
         # The minimum similarity between two nodes to consider them equal
         if F is None:
             F = 0.5
         self.F = F
-        # uniquattrs is a list of attributes that uniquely identifies a node
-        # inside a document. Defaults to 'xml:id'.
+        # uniqueattrs is a list of attributes or (tag, attribute) pairs
+        # that uniquely identifies a node inside a document. Defaults
+        # to 'xml:id'.
         if uniqueattrs is None:
-            uniqueattrs = ['{http://www.w3.org/XML/1998/namespace}id']
+            uniqueattrs = ["{http://www.w3.org/XML/1998/namespace}id"]
         self.uniqueattrs = uniqueattrs
         self.fast_match = fast_match
 
         # Avoid recreating this for every node
         self._sequencematcher = SequenceMatcher()
-        if ratio_mode == 'fast':
+        if ratio_mode == "fast":
             self._sequence_ratio = self._sequencematcher.quick_ratio
-        elif ratio_mode == 'accurate':
+        elif ratio_mode == "accurate":
             self._sequence_ratio = self._sequencematcher.ratio
-        elif ratio_mode == 'faster':
+        elif ratio_mode == "faster":
             self._sequence_ratio = self._sequencematcher.real_quick_ratio
         else:
             raise ValueError("Unknown ratio_mode '%s'" % ratio_mode)
 
+        self.ignored_attrs = ignored_attrs
+
         self.clear()
 
     def clear(self):
@@ -56,8 +63,9 @@ class Differ(object):
             right = right.getroot()
 
         if not (etree.iselement(left) and etree.iselement(right)):
-            raise TypeError("The 'left' and 'right' parameters must be "
-                            "lxml Elements.")
+            raise TypeError(
+                "The 'left' and 'right' parameters must be " "lxml Elements."
+            )
 
         # Left gets modified as a part of the diff, deepcopy it first.
         self.left = deepcopy(left)
@@ -110,14 +118,15 @@ class Differ(object):
 
         if self.fast_match:
             # First find matches with longest_common_subsequence:
-            matches = list(utils.longest_common_subsequence(
-                lnodes, rnodes, lambda x, y: self.node_ratio(x, y) >= 0.5))
+            matches = list(
+                utils.longest_common_subsequence(
+                    lnodes, rnodes, lambda x, y: self.node_ratio(x, y) >= 0.5
+                )
+            )
 
             # Add the matches (I prefer this from start to finish):
             for left_match, right_match in matches:
-                self.append_match(lnodes[left_match],
-                                  rnodes[right_match],
-                                  None)
+                self.append_match(lnodes[left_match], rnodes[right_match], None)
 
             # Then remove the nodes (needs to be done backwards):
             for left_match, right_match in reversed(matches):
@@ -149,7 +158,6 @@ class Differ(object):
 
         # Match the roots
         self.append_match(self.left, self.right, 1.0)
-
         return self._matches
 
     def node_ratio(self, left, right):
@@ -162,6 +170,12 @@ class Differ(object):
             return 0
 
         for attr in self.uniqueattrs:
+            if not isinstance(attr, str):
+                # If it's actually a sequence of (tag, attr), the tags must
+                # match first.
+                tag, attr = attr
+                if tag != left.tag or tag != right.tag:
+                    continue
             if attr in left.attrib or attr in right.attrib:
                 # One of the nodes have a unique attribute, we check only that.
                 # If only one node has it, it means they are not the same.
@@ -171,27 +185,36 @@ class Differ(object):
         child_ratio = self.child_ratio(left, right)
 
         if child_ratio is not None:
-            match = (match + child_ratio) / 2
+            match = sqrt((match**2 + child_ratio**2) / 2)
         return match
 
     def node_text(self, node):
         if node in self._text_cache:
             return self._text_cache[node]
         # Get the texts and the tag as a start
-        texts = node.xpath('text()')
+        texts = node.xpath("text()")
 
         # Then add attributes and values
-        for tag, value in sorted(node.attrib.items()):
-            if tag[0] == '{':
-                tag = tag.split('}',)[-1]
-            texts.append('%s:%s' % (tag, value))
+        for tag, value in sorted(self.node_attribs(node).items()):
+            if tag[0] == "{":
+                tag = tag.split(
+                    "}",
+                )[-1]
+            texts.append(f"{tag}:{value}")
 
         # Finally make one string, useful to see how similar two nodes are
-        text = u' '.join(texts).strip()
+        text = " ".join(texts).strip()
         result = utils.cleanup_whitespace(text)
         self._text_cache[node] = result
         return result
 
+    def node_attribs(self, node):
+        """Return a dict of attributes to consider for this node."""
+        attribs = dict(node.attrib)
+        for key in self.ignored_attrs:
+            attribs.pop(key, None)
+        return attribs
+
     def leaf_ratio(self, left, right):
         # How similar two nodes are, with no consideration of their children
         # We use a simple ratio here, I tried Levenshtein distances
@@ -230,8 +253,8 @@ class Differ(object):
 
         # Update: Look for differences in attributes
 
-        left_keys = set(left.attrib.keys())
-        right_keys = set(right.attrib.keys())
+        left_keys = set(self.node_attribs(left).keys())
+        right_keys = set(self.node_attribs(right).keys())
         new_keys = right_keys.difference(left_keys)
         removed_keys = left_keys.difference(right_keys)
         common_keys = left_keys.intersection(right_keys)
@@ -249,8 +272,7 @@ class Differ(object):
         # Move: Check if any of the new attributes have the same value
         # as the removed attributes. If they do, it's actually
         # a renaming, and a move is one action instead of remove + insert
-        newattrmap = {v: k for (k, v) in right.attrib.items()
-                      if k in new_keys}
+        newattrmap = {v: k for (k, v) in right.attrib.items() if k in new_keys}
         for lk in sorted(removed_keys):
             value = left.attrib[lk]
             if value in newattrmap:
@@ -258,6 +280,8 @@ class Differ(object):
                 yield actions.RenameAttrib(left_xpath, lk, rk)
                 # Remove from list of new attributes
                 new_keys.remove(rk)
+                # Delete used attribute from map of attributes
+                del newattrmap[value]
                 # Update left node
                 left.attrib[rk] = value
                 del left.attrib[lk]
@@ -323,19 +347,23 @@ class Differ(object):
         return i
 
     def align_children(self, left, right):
-        lchildren = [c for c in left.getchildren()
-                     if (id(c) in self._l2rmap and
-                         self._l2rmap[id(c)].getparent() is right)]
-        rchildren = [c for c in right.getchildren()
-                     if (id(c) in self._r2lmap and
-                         self._r2lmap[id(c)].getparent() is left)]
+        lchildren = [
+            c
+            for c in left.getchildren()
+            if (id(c) in self._l2rmap and self._l2rmap[id(c)].getparent() is right)
+        ]
+        rchildren = [
+            c
+            for c in right.getchildren()
+            if (id(c) in self._r2lmap and self._r2lmap[id(c)].getparent() is left)
+        ]
         if not lchildren or not rchildren:
             # Nothing to align
             return
 
         lcs = utils.longest_common_subsequence(
-            lchildren, rchildren,
-            lambda x, y: self._l2rmap[id(x)] is y)
+            lchildren, rchildren, lambda x, y: self._l2rmap[id(x)] is y
+        )
 
         for x, y in lcs:
             # Mark these as in order
@@ -345,7 +373,7 @@ class Differ(object):
         # Go over those children that are not in order:
         for lchild in lchildren:
             if lchild in self._inorder:
-                # Alrady aligned
+                # Already aligned
                 continue
 
             rchild = self._l2rmap[id(lchild)]
@@ -353,9 +381,8 @@ class Differ(object):
             rtarget = rchild.getparent()
             ltarget = self._r2lmap[id(rtarget)]
             yield actions.MoveNode(
-                utils.getpath(lchild),
-                utils.getpath(ltarget),
-                right_pos)
+                utils.getpath(lchild), utils.getpath(ltarget), right_pos
+            )
             # Do the actual move:
             left.remove(lchild)
             ltarget.insert(right_pos, lchild)
@@ -385,11 +412,13 @@ class Differ(object):
                 # (ii)
                 if rnode.tag is etree.Comment:
                     yield actions.InsertComment(
-                        utils.getpath(ltarget, ltree), pos, rnode.text)
+                        utils.getpath(ltarget, ltree), pos, rnode.text
+                    )
                     lnode = etree.Comment(rnode.text)
                 else:
-                    yield actions.InsertNode(utils.getpath(ltarget, ltree),
-                                             rnode.tag, pos)
+                    yield actions.InsertNode(
+                        utils.getpath(ltarget, ltree), rnode.tag, pos
+                    )
                     lnode = ltarget.makeelement(rnode.tag)
 
                     # (iii)
@@ -400,8 +429,7 @@ class Differ(object):
                 # And then we update attributes. This is different from the
                 # paper, because the paper assumes nodes only has labels and
                 # values. Nodes also has texts, we do them later.
-                for action in self.update_node_attr(lnode, rnode):
-                    yield action
+                yield from self.update_node_attr(lnode, rnode)
 
             # (c)
             else:
@@ -417,9 +445,8 @@ class Differ(object):
                 if ltarget is not lparent:
                     pos = self.find_pos(rnode)
                     yield actions.MoveNode(
-                        utils.getpath(lnode, ltree),
-                        utils.getpath(ltarget, ltree),
-                        pos)
+                        utils.getpath(lnode, ltree), utils.getpath(ltarget, ltree), pos
+                    )
                     # Move the node from current parent to target
                     lparent.remove(lnode)
                     ltarget.insert(pos, lnode)
@@ -427,26 +454,22 @@ class Differ(object):
                     self._inorder.add(rnode)
 
                 # Rename
-                for action in self.update_node_tag(lnode, rnode):
-                    yield action
+                yield from self.update_node_tag(lnode, rnode)
 
                 # (ii) Update
                 # XXX If they are exactly equal, we can skip this,
                 # maybe store match results in a cache?
-                for action in self.update_node_attr(lnode, rnode):
-                    yield action
+                yield from self.update_node_attr(lnode, rnode)
 
             # (d) Align
-            for action in self.align_children(lnode, rnode):
-                yield action
+            yield from self.align_children(lnode, rnode)
 
             # And lastly, we update all node texts. We do this after
             # aligning children, because when you generate an XML diff
             # from this, that XML diff update generates more children,
             # confusing later inserts or deletes.
             lnode = self._r2lmap[id(rnode)]
-            for action in self.update_node_text(lnode, rnode):
-                yield action
+            yield from self.update_node_text(lnode, rnode)
 
         for lnode in utils.reverse_post_order_traverse(self.left):
             if id(lnode) not in self._l2rmap:
